@@ -47,7 +47,6 @@ org.mozilla.jss.pkcs11.PK11RSAPublicKey
 org.mozilla.jss.pkcs11.PK11DSAPublicKey
 org.mozilla.jss.pkcs11.PK11SecureRandom 
 org.mozilla.jss.provider.java.security.JSSKeyStoreSpi
-org.mozilla.jss.SecretDecoderRing.KeyManager
 org.mozilla.jss.ssl.SSLSocket 
 org.mozilla.jss.ssl.SSLServerSocket 
 org.mozilla.jss.ssl.SocketBase 
@@ -71,7 +70,6 @@ org.mozilla.jss.pkix.cms
 org.mozilla.jss.pkix.crmf
 org.mozilla.jss.provider.java.security
 org.mozilla.jss.provider.javax.crypto
-org.mozilla.jss.SecretDecoderRing
 org.mozilla.jss.ssl
 org.mozilla.jss.tests
 org.mozilla.jss.util
@@ -120,32 +118,28 @@ sub setup_vars {
     $javadoc = "$ENV{JAVA_HOME}/bin/javadoc";
 
     $dist_dir = $cmdline_vars{SOURCE_PREFIX};
+    $ENV{JCE_JAR} or die "Must specify JCE_JAR environment variable";
     $jce_jar = $ENV{JCE_JAR};
 
     $class_release_dir = $cmdline_vars{SOURCE_RELEASE_PREFIX};
     if( $ENV{BUILD_OPT} ) {
         $class_dir = "$dist_dir/classes";
-        $class_jar = "$dist_dir/$cmdline_vars{XPCLASS_JAR}";
         $class_release_dir .= "/$cmdline_vars{SOURCE_RELEASE_CLASSES_DIR}";
         $javac_opt_flag = "-O";
         $debug_source_file = "org/mozilla/jss/util/Debug_ship.jnot";
     } else {
         $class_dir = "$dist_dir/classes_DBG";
-        $class_jar = "$dist_dir/$cmdline_vars{XPCLASS_DBG_JAR}";
         $class_release_dir .= "/$cmdline_vars{SOURCE_RELEASE_CLASSES_DBG_DIR}";
         $javac_opt_flag = "-g";
         $debug_source_file = "org/mozilla/jss/util/Debug_debug.jnot";
     }
     $jni_header_dir = "$dist_dir/private/jss/_jni";
 
-    if( $jce_jar ) {
-        $classpath = "-classpath $jce_jar";
-    }
+    $classpath = "$jce_jar";
 }
 
 sub clean {
     print_do("rm -rf $class_dir");
-    print_do("rm -f $class_jar");
     print_do("rm -rf $jni_header_dir");
 }
 
@@ -194,8 +188,7 @@ sub build {
     if( scalar(@source_list) > 0 ) {
         ensure_dir_exists($class_dir);
         print_do("$javac $javac_opt_flag -sourcepath . -d $class_dir " .
-            "$classpath " . join(" ",@source_list));
-        print_do("cd $class_dir; rm -f $class_jar; zip -r $class_jar .");
+            "-classpath $classpath " . join(" ",@source_list));
     }
 
     #
@@ -288,7 +281,7 @@ sub javadoc {
     ensure_dir_exists("$dist_dir/jssdoc");
     my $targets = join(" ", @packages);
     print "$targets\n";
-    print_do("$javadoc -breakiterator -sourcepath . -d $dist_dir/jssdoc $html_header_opt $targets");
-    print_do("cp $dist_dir/jssdoc/index.html $dist_dir/jssdoc/index.html.bak");
+    print_do("$javadoc -private -breakiterator -sourcepath . -d $dist_dir/jssdoc $html_header_opt $targets");
+    #print "$javadoc -private -sourcepath . -d $dist_dir/jssdoc $html_header_opt $targets" . "\n";
     print_do("cp $dist_dir/jssdoc/overview-summary.html $dist_dir/jssdoc/index.html");
 }
