@@ -377,8 +377,9 @@ public static class Template implements ASN1Template {
                     // use the default
                     seq.addElement( e.getDefault() );
                 } else {
-                    throw new InvalidBERException("Missing item #"+index+
-                        " need tag " + lookAhead.getTag() + " element tag " + e.getImplicitTag() + " in SEQUENCE");
+                    throw new InvalidBERException("Missing item #" + index +
+                        ": expecting " + lookAhead.getTag() + ", found"
+                        + e.getImplicitTag() );
                 }
                 continue;
             }
@@ -422,9 +423,10 @@ public static class Template implements ASN1Template {
         }
 
         if( remainingContent > 0 ) {
-            throw new InvalidBERException("SEQUENCE is longer than expected "+remainingContent+" expected");
+            throw new InvalidBERException("SEQUENCE is " + remainingContent +
+                " bytes longer than expected");
         }
-        Assert.assert( remainingContent == 0 || remainingContent == -1 );
+        Assert._assert( remainingContent == 0 || remainingContent == -1 );
 
         // If this was indefinite-length encoding, consume the end-of-contents
         if( remainingContent == -1 ) {
@@ -436,7 +438,7 @@ public static class Template implements ASN1Template {
 
         // Make sure we stayed in sync
         if( ! repeatableElement ) {
-            Assert.assert(index == seq.size());
+            Assert._assert(index == seq.size());
         }
 
         return seq;
