@@ -45,7 +45,7 @@ import java.security.InvalidParameterException;
  * CryptoManager class.
  *
  * @author nicolson
- * @version $Revision: 1.2 $ $Date: 2000/12/19 06:18:13 $ 
+ * @version $Revision: 1.5 $ $Date: 2002/07/15 22:47:53 $ 
  * @see org.mozilla.jss.CryptoManager
  */
 public final class PK11Token implements CryptoToken {
@@ -69,20 +69,20 @@ public final class PK11Token implements CryptoToken {
     ////////////////////////////////////////////////////
 	public org.mozilla.jss.crypto.Signature
 	getSignatureContext(SignatureAlgorithm algorithm) 
-		throws java.security.NoSuchAlgorithmException, TokenException
+		throws NoSuchAlgorithmException, TokenException
 	{
-        Assert.assert(algorithm!=null);
+        Assert._assert(algorithm!=null);
 		return Tunnel.constructSignature( algorithm,
                     new PK11Signature(this, algorithm) );
 	}
 
 	public JSSMessageDigest
 	getDigestContext(DigestAlgorithm algorithm)
-		throws java.security.NoSuchAlgorithmException,
+		throws NoSuchAlgorithmException,
                 java.security.DigestException
 	{
         if( ! doesAlgorithm(algorithm) ) {
-            throw new java.security.NoSuchAlgorithmException();
+            throw new NoSuchAlgorithmException();
         }
 
         return new PK11MessageDigest(this, algorithm);
@@ -90,10 +90,10 @@ public final class PK11Token implements CryptoToken {
 
 	public Cipher
 	getCipherContext(EncryptionAlgorithm algorithm)
-		throws java.security.NoSuchAlgorithmException, TokenException
+		throws NoSuchAlgorithmException, TokenException
 	{
         if( ! doesAlgorithm(algorithm) ) {
-            throw new java.security.NoSuchAlgorithmException(
+            throw new NoSuchAlgorithmException(
                 algorithm+" is not supported by this token");
         }
         return new PK11Cipher(this, algorithm);
@@ -101,10 +101,10 @@ public final class PK11Token implements CryptoToken {
 
     public KeyGenerator
     getKeyGenerator(KeyGenAlgorithm algorithm)
-        throws java.security.NoSuchAlgorithmException, TokenException
+        throws NoSuchAlgorithmException, TokenException
     {
         if( ! doesAlgorithm(algorithm) ) {
-            throw new java.security.NoSuchAlgorithmException(
+            throw new NoSuchAlgorithmException(
                 algorithm+" is not supported by this token");
         }
         return new PK11KeyGenerator(this, algorithm);
@@ -127,10 +127,10 @@ public final class PK11Token implements CryptoToken {
 
     public KeyWrapper
     getKeyWrapper(KeyWrapAlgorithm algorithm)
-        throws java.security.NoSuchAlgorithmException, TokenException
+        throws NoSuchAlgorithmException, TokenException
     {
         if( ! doesAlgorithm(algorithm) ) {
-            throw new java.security.NoSuchAlgorithmException(
+            throw new NoSuchAlgorithmException(
                 algorithm+" is not supported by this token");
         }
         return new PK11KeyWrapper(this, algorithm);
@@ -145,9 +145,9 @@ public final class PK11Token implements CryptoToken {
 
 	public org.mozilla.jss.crypto.KeyPairGenerator
 	getKeyPairGenerator(KeyPairAlgorithm algorithm)
-		throws java.security.NoSuchAlgorithmException, TokenException
+		throws NoSuchAlgorithmException, TokenException
 	{
-        Assert.assert(algorithm!=null);
+        Assert._assert(algorithm!=null);
         return new KeyPairGenerator(algorithm,
                                     new PK11KeyPairGenerator(this, algorithm));
 	}
@@ -449,7 +449,7 @@ public final class PK11Token implements CryptoToken {
      * @param P The DSA prime parameter
      * @param Q The DSA sub-prime parameter
      * @param G The DSA base parameter
-	 * @ return String that represents a PKCS#10 b64 encoded blob with
+	 * @return String that represents a PKCS#10 b64 encoded blob with
 	 * begin/end brackets
 	 */
 	public String generateCertRequest(String subject, int keysize,
@@ -524,7 +524,7 @@ public final class PK11Token implements CryptoToken {
      * Default constructor should never be called.
      */
     protected PK11Token() {
-        Assert.assert(false);
+        Assert._assert(false);
     }
 
     /**
@@ -533,7 +533,7 @@ public final class PK11Token implements CryptoToken {
      * @param pointer A byte array containing a pointer to a PKCS #11 slot.
      */
     protected PK11Token(byte[] pointer, boolean internal, boolean keyStorage) {
-        Assert.assert(pointer!=null);
+        Assert._assert(pointer!=null);
         tokenProxy = new TokenProxy(pointer);
         mIsInternalCryptoToken = internal;
         mIsInternalKeyStorageToken = keyStorage;
@@ -542,12 +542,12 @@ public final class PK11Token implements CryptoToken {
 
 /*
 	protected PK11Token(TokenProxy proxy) {
-        Assert.assert(proxy!=null);
+        Assert._assert(proxy!=null);
 		this.tokenProxy = proxy;
 	}
 */
 
-	TokenProxy getProxy() {
+	public TokenProxy getProxy() {
 		return tokenProxy;
 	}
 
