@@ -321,7 +321,7 @@ public class SignerInfo implements ASN1Value {
 
         if( authenticatedAttributes != null ) 
         {
-            Assert.assert( authenticatedAttributes.size() >= 2 );
+            Assert._assert( authenticatedAttributes.size() >= 2 );
             this.authenticatedAttributes = authenticatedAttributes;
         }
 
@@ -738,7 +738,7 @@ public class SignerInfo implements ASN1Value {
      */
     private static boolean byteArraysAreSame(byte[] left, byte[] right) {
 
-        Assert.assert(left!=null && right!=null);
+        Assert._assert(left!=null && right!=null);
 
         if( left.length != right.length ) {
             return false;
@@ -843,7 +843,7 @@ public class SignerInfo implements ASN1Value {
             throws IOException, InvalidBERException
             {
                 SEQUENCE seq = (SEQUENCE) seqt.decode(implicitTag,istream);
-                Assert.assert(seq.size() == 7);
+                Assert._assert(seq.size() == 7);
 
                 return new SignerInfo(
                     (INTEGER)                   seq.elementAt(0),
