@@ -37,7 +37,6 @@ import java.io.*;
 import org.mozilla.jss.asn1.*;
 import org.mozilla.jss.util.Assert;
 import org.mozilla.jss.pkix.primitive.*;
-import org.mozilla.jss.pkix.cert.*;
 import org.mozilla.jss.crypto.*;
 import java.util.Vector;
 import java.math.BigInteger;
@@ -47,6 +46,8 @@ import java.security.SignatureException;
 import java.security.NoSuchAlgorithmException;
 import java.security.MessageDigest;
 import org.mozilla.jss.crypto.*;
+import org.mozilla.jss.crypto.X509Certificate;
+import org.mozilla.jss.pkix.cert.*;
 import org.mozilla.jss.*;
 import java.security.PublicKey;
 
@@ -321,7 +322,7 @@ public class SignerInfo implements ASN1Value {
 
         if( signedAttributes != null ) 
         {
-            Assert.assert( signedAttributes.size() >= 2 );
+            Assert._assert( signedAttributes.size() >= 2 );
             this.signedAttributes = signedAttributes;
         }
 
@@ -444,7 +445,7 @@ public class SignerInfo implements ASN1Value {
 				issuerAndSerialNumber.getSerialNumber()  );
 			verify(messageDigest, contentType, cert.getPublicKey());
 		} else {
-            Assert.assert(
+            Assert._assert(
 						  signerIdentifier.getType().equals(SignerIdentifier.SUBJECT_KEY_IDENTIFIER) );
 
 			// xxxxx Do we have method to get key using keyIdentifier
@@ -749,7 +750,7 @@ public class SignerInfo implements ASN1Value {
      */
     private static boolean byteArraysAreSame(byte[] left, byte[] right) {
 
-        Assert.assert(left!=null && right!=null);
+        Assert._assert(left!=null && right!=null);
 
         if( left.length != right.length ) {
             return false;
@@ -854,7 +855,7 @@ public class SignerInfo implements ASN1Value {
             throws IOException, InvalidBERException
             {
                 SEQUENCE seq = (SEQUENCE) seqt.decode(implicitTag,istream);
-                Assert.assert(seq.size() == 7);
+                Assert._assert(seq.size() == 7);
 
                 return new SignerInfo(
                     (INTEGER)                   seq.elementAt(0),
