@@ -37,6 +37,9 @@ import java.net.InetAddress;
 import java.io.IOException;
 import java.net.Socket;
 import java.net.SocketException;
+import org.mozilla.jss.CryptoManager;
+import org.mozilla.jss.crypto.ObjectNotFoundException;
+import org.mozilla.jss.crypto.TokenException;
 
 /**
  * SSL server socket.
@@ -172,8 +175,8 @@ public class SSLServerSocket extends java.net.ServerSocket {
         return base.getTimeout();
     }
 
-    protected native void setReuseAddress(boolean reuse) throws SocketException;
-    protected native boolean getReuseAddress() throws SocketException;
+    public native void setReuseAddress(boolean reuse) throws SocketException;
+    public native boolean getReuseAddress() throws SocketException;
 
     private native byte[] socketAccept(SSLSocket s, int timeout,
         boolean handshakeAsClient) throws SocketException;
@@ -183,18 +186,21 @@ public class SSLServerSocket extends java.net.ServerSocket {
      */
     public static native void clearSessionCache();
 
-    protected void finalize() throws Throwable {
-        close();
+    protected void finalize() throws Throwable { }
+
+
+    /**
+     * @return The local port.
+     */
+    public int getLocalPort() {
+        return base.getLocalPort();
     }
 
     /**
      * Closes this socket.
      */
     public void close() throws IOException {
-        if( sockProxy != null ) {
-            base.close();
-            sockProxy = null;
-        }
+        base.close();
     }
 
     // This directory is used as the default for the Session ID cache
@@ -223,8 +229,25 @@ public class SSLServerSocket extends java.net.ServerSocket {
     /**
      * Sets the certificate to use for server authentication.
      */
-    public native void setServerCertNickname(String nickname)
-            throws SocketException;
+    public void setServerCertNickname(String nick) throws SocketException
+    {
+      try {
+        setServerCert( CryptoManager.getInstance().findCertByNickname(nick) );
+      } catch(CryptoManager.NotInitializedException nie) {
+        throw new SocketException("CryptoManager not initialized");
+      } catch(ObjectNotFoundException onfe) {
+        throw new SocketException("Object not found: " + onfe);
+      } catch(TokenException te) {
+        throw new SocketException("Token Exception: " + te);
+      }
+    }
+
+    /**
+     * Sets the certificate to use for server authentication.
+     */
+    public native void setServerCert(
+        org.mozilla.jss.crypto.X509Certificate certnickname)
+        throws SocketException;
 
     /**
      * Enables/disables the request of client authentication. This is only
@@ -284,10 +307,10 @@ public class SSLServerSocket extends java.net.ServerSocket {
     }
 
     /**
-     * @return The remote peer's IP address.
+     * @return the local address of this server socket.
      */
     public InetAddress getInetAddress() {
-        return base.getInetAddress();
+        return base.getLocalAddress();
     }
 
     /**
@@ -309,6 +332,15 @@ public class SSLServerSocket extends java.net.ServerSocket {
         base.setClientCertNickname(nick);
     }
 
+    /**
+     * Sets the certificate to use for client authentication.
+     */
+    public void setClientCert(org.mozilla.jss.crypto.X509Certificate cert)
+        throws SocketException
+    {
+        base.setClientCert(cert);
+    }
+
     /**
      * Determines whether this end of the socket is the client or the server
      *  for purposes of the SSL protocol. By default, it is the server.
@@ -326,4 +358,17 @@ public class SSLServerSocket extends java.net.ServerSocket {
     public void useCache(boolean b) throws SocketException {
         base.useCache(b);
     }
+
+    /**
+     * Returns the addresses and ports of this socket.
+     */
+    public String toString() {
+        StringBuffer buf = new StringBuffer();
+        buf.append("SSLServerSocket[addr=");
+        buf.append(getInetAddress());
+        buf.append(",port=0,localport=");
+        buf.append(getLocalPort());
+        buf.append("]");
+        return buf.toString();
+    }
 }
