@@ -57,7 +57,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setSSLDefaultOption(JNIEnv *env,
     /* set the option */
     status = SSL_OptionSetDefault(JSSL_enums[joption], on);
     if( status != SECSuccess ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "SSL_OptionSet failed");
+        JSSL_throwSSLSocketException(env, "SSL_OptionSet failed");
         goto finish;
     }
 
@@ -65,10 +65,17 @@ finish:
     return;
 }
 
+#if 0
+#define EXCEPTION_CHECK(env, sock) \
+    if( sock != NULL && sock->jsockPriv!=NULL) { \
+        JSS_SSL_processExceptions(env, sock->jsockPriv); \
+    }
+#endif
+
 JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_forceHandshake(JNIEnv *env, jobject self)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     int rv;
 
     /* get my fd */
@@ -77,17 +84,18 @@ Java_org_mozilla_jss_ssl_SSLSocket_forceHandshake(JNIEnv *env, jobject self)
     /* do the work */
     rv = SSL_ForceHandshake(sock->fd);
     if( rv != SECSuccess ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "SSL_ForceHandshake failed");
+        JSSL_throwSSLSocketException(env, "SSL_ForceHandshake failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
 /*
  * linger
- *      The linger time, in hundredths of a second.
+ *      The linger time, in seconds.
  */
 JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_setSoLinger(JNIEnv *env, jobject self,
@@ -95,7 +103,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setSoLinger(JNIEnv *env, jobject self,
 {
     PRSocketOptionData sockOptions;
     PRStatus status;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -104,18 +112,18 @@ Java_org_mozilla_jss_ssl_SSLSocket_setSoLinger(JNIEnv *env, jobject self,
     sockOptions.option = PR_SockOpt_Linger;
     sockOptions.value.linger.polarity = on;
     if(on) {
-        sockOptions.value.linger.linger =
-            PR_MillisecondsToInterval(linger * 10);
+        sockOptions.value.linger.linger = PR_SecondsToInterval(linger);
     }
 
     status = PR_SetSocketOption(sock->fd, &sockOptions);
 
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_SetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_SetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -123,7 +131,7 @@ JNIEXPORT jboolean JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_getTcpNoDelay(JNIEnv *env, jobject self)
 {
     PRSocketOptionData sockOptions;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     PRStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
@@ -134,11 +142,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_getTcpNoDelay(JNIEnv *env, jobject self)
 
     status = PR_GetSocketOption(sock->fd, &sockOptions);
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_GetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_GetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return sockOptions.value.no_delay;
 }
 
@@ -148,7 +157,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setTcpNoDelay(JNIEnv *env, jobject self,
 {
     PRSocketOptionData sockOptions;
     PRStatus status;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -160,11 +169,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_setTcpNoDelay(JNIEnv *env, jobject self,
     status = PR_SetSocketOption(sock->fd, &sockOptions);
 
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_SetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_SetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -172,7 +182,7 @@ JNIEXPORT jint JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_getSendBufferSize(JNIEnv *env, jobject self)
 {
     PRSocketOptionData sockOptions;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     PRStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
@@ -183,11 +193,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_getSendBufferSize(JNIEnv *env, jobject self)
 
     status = PR_GetSocketOption(sock->fd, &sockOptions);
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_GetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_GetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return sockOptions.value.send_buffer_size;
 }
 
@@ -197,7 +208,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setSendBufferSize(JNIEnv *env, jobject self,
 {
     PRSocketOptionData sockOptions;
     PRStatus status;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -209,11 +220,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_setSendBufferSize(JNIEnv *env, jobject self,
     status = PR_SetSocketOption(sock->fd, &sockOptions);
 
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_SetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_SetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -221,7 +233,7 @@ JNIEXPORT jboolean JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_getKeepAlive(JNIEnv *env, jobject self)
 {
     PRSocketOptionData sockOptions;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     PRStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
@@ -232,11 +244,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_getKeepAlive(JNIEnv *env, jobject self)
 
     status = PR_GetSocketOption(sock->fd, &sockOptions);
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_GetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_GetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return sockOptions.value.keep_alive;
 }
 
@@ -245,7 +258,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_getReceiveBufferSize(
     JNIEnv *env, jobject self)
 {
     PRSocketOptionData sockOptions;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     PRStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
@@ -256,11 +269,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_getReceiveBufferSize(
 
     status = PR_GetSocketOption(sock->fd, &sockOptions);
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_GetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_GetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return sockOptions.value.recv_buffer_size;
 }
 
@@ -270,7 +284,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setReceiveBufferSize(
 {
     PRSocketOptionData sockOptions;
     PRStatus status;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -282,11 +296,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_setReceiveBufferSize(
     status = PR_SetSocketOption(sock->fd, &sockOptions);
 
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_SetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_SetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -296,7 +311,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setKeepAlive(JNIEnv *env, jobject self,
 {
     PRSocketOptionData sockOptions;
     PRStatus status;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -308,11 +323,12 @@ Java_org_mozilla_jss_ssl_SSLSocket_setKeepAlive(JNIEnv *env, jobject self,
     status = PR_SetSocketOption(sock->fd, &sockOptions);
 
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_SetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_SetSocketOption failed");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -320,7 +336,7 @@ JNIEXPORT jint JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_getSoLinger(JNIEnv *env, jobject self)
 {
     PRSocketOptionData sockOptions;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     jint retval;
     PRStatus status;
 
@@ -332,28 +348,36 @@ Java_org_mozilla_jss_ssl_SSLSocket_getSoLinger(JNIEnv *env, jobject self)
 
     status = PR_GetSocketOption(sock->fd, &sockOptions);
     if( status != PR_SUCCESS ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "PR_GetSocketOption failed");
+        JSSL_throwSSLSocketException(env, "PR_GetSocketOption failed");
         goto finish;
     }
 
     if( sockOptions.value.linger.polarity == PR_TRUE ) {
-        retval = PR_IntervalToMilliseconds(sockOptions.value.linger.linger) /10;
+        retval = PR_IntervalToSeconds(sockOptions.value.linger.linger);
     } else {
         retval = -1;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return retval;
 }
 
+/*
+ * This function is only here for binary compatibility. See
+ * http://bugzilla.mozilla.org/show_bug.cgi?id=143254
+ */
 JNIEXPORT jint JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_getLocalAddressNative(JNIEnv *env,
     jobject self)
 {
     PRNetAddr addr;
 
-    JSSL_getSockAddr(env, self, &addr, LOCAL_SOCK);
-    return ntohl(addr.inet.ip);
+    if( JSSL_getSockAddr(env, self, &addr, LOCAL_SOCK) == PR_SUCCESS ) {
+        return ntohl(addr.inet.ip);
+    } else {
+        return 0;
+    }
 }
 
 JNIEXPORT jint JNICALL
@@ -362,8 +386,11 @@ Java_org_mozilla_jss_ssl_SSLSocket_getPort(JNIEnv *env,
 {
     PRNetAddr addr;
 
-    JSSL_getSockAddr(env, self, &addr, PEER_SOCK);
-    return addr.inet.port;
+    if( JSSL_getSockAddr(env, self, &addr, PEER_SOCK) == PR_SUCCESS ) {
+        return ntohs(addr.inet.port);
+    } else {
+        return 0;
+    }
 }
 
 JNIEXPORT void JNICALL
@@ -404,7 +431,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketConnect
     if( hostnameStr == NULL ) goto finish;
     stat = SSL_SetURL(sock->fd, (char*)hostnameStr);
     if( stat != 0 ) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to set the SSL URL");
+        JSSL_throwSSLSocketException(env, "Failed to set the SSL URL");
         goto finish;
     }
 
@@ -413,11 +440,14 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketConnect
      */
     status = PR_Connect(sock->fd, &addr, PR_INTERVAL_NO_TIMEOUT);
     if( status != PR_SUCCESS) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Unable to connect");
+        JSSL_throwSSLSocketException(env, "Unable to connect");
         goto finish;
     }
 
 finish:
+    /* This method should never be called on a Java socket wrapper. */
+    PR_ASSERT( sock==NULL || sock->jsockPriv==NULL);
+
     if( hostnameStr != NULL ) {
         (*env)->ReleaseStringUTFChars(env, hostname, hostnameStr);
     }
@@ -463,7 +493,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_getStatus
                                     &subject);
 
     if(secstatus != SECSuccess) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION,
+        JSSL_throwSSLSocketException(env,
             "Failed to retrieve socket security status");
         goto finish;
     }
@@ -535,6 +565,7 @@ finish:
         PR_Free(serialNum);
     }
 
+    EXCEPTION_CHECK(env, sock)
     return statusObj;
 }
 
@@ -551,7 +582,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setCipherPreference(
         char buf[128];
         PR_snprintf(buf, 128, "Failed to %s cipher 0x%lx\n",
             (enable ? "enable" : "disable"), cipher);
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, buf);
+        JSSL_throwSSLSocketException(env, buf);
         goto finish;
     }
 
@@ -563,7 +594,7 @@ JNIEXPORT jint JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_socketRead(JNIEnv *env, jobject self, 
     jbyteArray bufBA, jint off, jint len, jint timeout)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     jbyte *buf = NULL;
     jint size;
     PRIntervalTime ivtimeout;
@@ -604,7 +635,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketRead(JNIEnv *env, jobject self,
                 /* just try again */
             } else {
                 /* unrecoverable error */
-                JSS_throwMsgPrErr(env, SOCKET_EXCEPTION,
+                JSSL_throwSSLSocketException(env,
                     "Error reading from socket");
                 goto finish;
             }
@@ -617,8 +648,9 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketRead(JNIEnv *env, jobject self,
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     (*env)->ReleaseByteArrayElements(env, bufBA, buf,
-        (nread>0) ? JNI_COMMIT : JNI_ABORT);
+        (nread>0) ? 0 /*copy and free*/ : JNI_ABORT /*free, no copy*/);
     return nread;
 }
 
@@ -627,7 +659,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketAvailable(
     JNIEnv *env, jobject self)
 {
     jint available;
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS ) {
         goto finish;
@@ -637,6 +669,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketAvailable(
     PR_ASSERT(available >= 0);
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return available;
 }
 
@@ -644,7 +677,7 @@ JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_socketWrite(JNIEnv *env, jobject self, 
     jbyteArray bufBA, jint off, jint len, jint timeout)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     jbyte *buf = NULL;
     jint size;
     PRIntervalTime ivtimeout;
@@ -684,10 +717,10 @@ Java_org_mozilla_jss_ssl_SSLSocket_socketWrite(JNIEnv *env, jobject self,
             {
                 /* just try again */
             } else if( err == PR_IO_TIMEOUT_ERROR ) {
-                JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Operation timed out");
+                JSSL_throwSSLSocketException(env, "Operation timed out");
                 goto finish;
             } else {
-                JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, 
+                JSSL_throwSSLSocketException(env,
                     "Failed to write to socket");
                 goto finish;
             }
@@ -702,42 +735,45 @@ finish:
     if( buf != NULL ) {
         (*env)->ReleaseByteArrayElements(env, bufBA, buf, JNI_ABORT);
     }
+    EXCEPTION_CHECK(env, sock)
 }
 
 JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_shutdownNative(
     JNIEnv *env, jobject self, jint how)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     PRStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS) goto finish;
 
     status = PR_Shutdown(sock->fd, JSSL_enums[how]);
     if( status != PR_SUCCESS) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to shutdown socket");
+        JSSL_throwSSLSocketException(env, "Failed to shutdown socket");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
 JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_invalidateSession(JNIEnv *env, jobject self)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     SECStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS) goto finish;
 
     status = SSL_InvalidateSession(sock->fd);
     if(status != SECSuccess) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to invalidate session");
+        JSSL_throwSSLSocketException(env, "Failed to invalidate session");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -745,18 +781,19 @@ JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_redoHandshake(
     JNIEnv *env, jobject self, jboolean flushCache)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     SECStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS) goto finish;
 
     status = SSL_ReHandshake(sock->fd, flushCache);
     if(status != SECSuccess) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to redo handshake");
+        JSSL_throwSSLSocketException(env, "Failed to redo handshake");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -764,18 +801,19 @@ JNIEXPORT void JNICALL
 Java_org_mozilla_jss_ssl_SSLSocket_resetHandshakeNative(
     JNIEnv *env, jobject self, jboolean asClient)
 {
-    JSSL_SocketData *sock;
+    JSSL_SocketData *sock = NULL;
     SECStatus status;
 
     if( JSSL_getSockData(env, self, &sock) != PR_SUCCESS) goto finish;
 
     status = SSL_ResetHandshake(sock->fd, !asClient);
     if(status != SECSuccess) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to redo handshake");
+        JSSL_throwSSLSocketException(env, "Failed to redo handshake");
         goto finish;
     }
 
 finish:
+    EXCEPTION_CHECK(env, sock)
     return;
 }
 
@@ -801,7 +839,7 @@ Java_org_mozilla_jss_ssl_SSLSocket_setCipherPolicyNative(
     }
 
     if(status != SECSuccess) {
-        JSS_throwMsgPrErr(env, SOCKET_EXCEPTION, "Failed to set cipher policy");
+        JSSL_throwSSLSocketException(env, "Failed to set cipher policy");
         goto finish;
     }
 
