@@ -230,6 +230,35 @@ public class SSLSocket extends java.net.Socket {
         socketConnect(address.getAddress(), hostname, port);
     }
 
+    /**
+     * Creates an SSL client socket using the given Java socket for underlying
+     *  I/O. Installs the given callbacks for certificate approval and
+     *  client certificate selection.
+     *
+     * @param s The Java socket to use for underlying I/O.
+     * @param host The hostname of the remote side of the connection.
+     *      This name is used to verify the server's certificate.
+     * @param certApprovalCallback A callback that can be used to override
+     *      approval of the peer's certificate.
+     * @param clientCertSelectionCallback A callback to select the client
+     *      certificate to present to the peer.
+     */
+    public SSLSocket(java.net.Socket s, String host,
+        SSLCertificateApprovalCallback certApprovalCallback,
+        SSLClientCertificateSelectionCallback clientCertSelectionCallback)
+            throws IOException
+    {
+        // create the socket
+        sockProxy =
+            new SocketProxy(
+                base.socketCreate(
+                    this, certApprovalCallback, clientCertSelectionCallback,
+                    s, host ) );
+
+        base.setProxy(sockProxy);
+        resetHandshake();
+    }
+
     /**
      * @return The remote peer's IP address.
      */
@@ -241,27 +270,8 @@ public class SSLSocket extends java.net.Socket {
      * @return The local IP address.
      */
     public InetAddress getLocalAddress() {
-        try {
-            int intAddr = getLocalAddressNative();
-            InetAddress in;
-            byte[] addr = new byte[4];
-            addr[0] = (byte)((intAddr >>> 24) & 0xff);
-            addr[1] = (byte)((intAddr >>> 16) & 0xff);
-            addr[2] = (byte)((intAddr >>>  8) & 0xff);
-            addr[3] = (byte)((intAddr       ) & 0xff);
-            try {
-            in = InetAddress.getByName(
-                addr[0] + "." + addr[1] + "." + addr[2] + "." + addr[3] );
-            } catch (java.net.UnknownHostException e) {
-                in = null;
-            }
-            return in;
-        } catch(SocketException e) {
-            e.printStackTrace();
-            return null;
-        }
+        return base.getLocalAddress();
     }
-    private native int getLocalAddressNative() throws SocketException;
 
     /**
      * @return The local port.
@@ -328,7 +338,7 @@ public class SSLSocket extends java.net.Socket {
 
     /**
      * Sets the SO_LINGER socket option.
-     * param linger The time (in hundredths of a second) to linger for.
+     * param linger The time (in seconds) to linger for.
      */
     public native void setSoLinger(boolean on, int linger)
         throws SocketException;
@@ -376,10 +386,7 @@ public class SSLSocket extends java.net.Socket {
      * Closes this socket.
      */
     public void close() throws IOException {
-        if( sockProxy != null ) {
-            base.close();
-            sockProxy = null;
-        }
+        base.close();
     }
 
     private native void socketConnect(byte[] addr, String hostname, int port)
@@ -514,11 +521,29 @@ public class SSLSocket extends java.net.Socket {
 
     /**
      * Sets the nickname of the certificate to use for client authentication.
+     * Alternately, you can specify an SSLClientCertificateSelectionCallback,
+     * which will receive a list of certificates that are valid for client
+     * authentication.
+     * @see org.mozilla.jss.ssl.SSLClientCertificateSelectionCallback
      */
     public void setClientCertNickname(String nick) throws SocketException {
         base.setClientCertNickname(nick);
     }
 
+    /**
+     * Sets the certificate to use for client authentication.
+     * Alternately, you can specify an SSLClientCertificateSelectionCallback,
+     * which will receive a list of certificates that are valid for client
+     * authentication.
+     * @see org.mozilla.jss.ssl.SSLClientCertificateSelectionCallback
+     */
+    public void setClientCert( org.mozilla.jss.crypto.X509Certificate cert)
+        throws SocketException
+    {
+        base.setClientCert(cert);
+    }
+
+
     /**
      * Enables/disables the request of client authentication. This is only
      *  meaningful for the server end of the SSL connection. During the next
@@ -638,9 +663,7 @@ public class SSLSocket extends java.net.Socket {
      */
     public native void redoHandshake(boolean flushCache) throws SocketException;
 
-    protected void finalize() throws Throwable {
-        close();
-    }
+    protected void finalize() throws Throwable { }
 
     public static class CipherPolicy {
         private int enum;
@@ -668,6 +691,22 @@ public class SSLSocket extends java.net.Socket {
     private static native void setCipherPolicyNative(int policyEnum)
         throws SocketException;
 
+    /**
+     * Returns the addresses and ports of this socket.
+     */
+    public String toString() {
+        StringBuffer buf = new StringBuffer();
+        buf.append("SSLSocket[addr=");
+        buf.append(getInetAddress());
+        buf.append(getLocalAddress());
+        buf.append(",port=");
+        buf.append(getPort());
+        buf.append(",localport=");
+        buf.append(getLocalPort());
+        buf.append("]");
+        return buf.toString();
+    }
+
     public final static int SSL2_RC4_128_WITH_MD5                  = 0xFF01;
     public final static int SSL2_RC4_128_EXPORT40_WITH_MD5         = 0xFF02;
     public final static int SSL2_RC2_128_CBC_WITH_MD5              = 0xFF03;
@@ -721,5 +760,20 @@ public class SSLSocket extends java.net.Socket {
     public final static int TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA  = 0x0065;
     public final static int TLS_DHE_DSS_WITH_RC4_128_SHA            = 0x0066;
 
+// New TLS cipher suites in NSS 3.4 
+    public final static int TLS_RSA_WITH_AES_128_CBC_SHA          =  0x002F;
+    public final static int TLS_DH_DSS_WITH_AES_128_CBC_SHA       =  0x0030;
+    public final static int TLS_DH_RSA_WITH_AES_128_CBC_SHA       =  0x0031;
+    public final static int TLS_DHE_DSS_WITH_AES_128_CBC_SHA      =  0x0032;
+    public final static int TLS_DHE_RSA_WITH_AES_128_CBC_SHA      =  0x0033;
+    public final static int TLS_DH_ANON_WITH_AES_128_CBC_SHA      =  0x0034;
+
+    public final static int TLS_RSA_WITH_AES_256_CBC_SHA          =  0x0035;
+    public final static int TLS_DH_DSS_WITH_AES_256_CBC_SHA       =  0x0036;
+    public final static int TLS_DH_RSA_WITH_AES_256_CBC_SHA       =  0x0037;
+    public final static int TLS_DHE_DSS_WITH_AES_256_CBC_SHA      =  0x0038;
+    public final static int TLS_DHE_RSA_WITH_AES_256_CBC_SHA      =  0x0039;
+    public final static int TLS_DH_ANON_WITH_AES_256_CBC_SHA      =  0x003A;
+
 }
 
