@@ -226,6 +226,10 @@ JSS_getPtrFromProxy(JNIEnv *env, jobject nativeProxy, void **ptr)
     int size;
 
     PR_ASSERT(env!=NULL && nativeProxy != NULL && ptr != NULL);
+    if( nativeProxy == NULL ) {
+        JSS_throw(env, NULL_POINTER_EXCEPTION);
+        return PR_FAILURE;
+    }
 
 	proxyClass = (*env)->GetObjectClass(env, nativeProxy);
 	PR_ASSERT(proxyClass != NULL);
@@ -666,7 +670,19 @@ JNIEXPORT jobject JNICALL Java_org_mozilla_jss_util_Password_readPasswordFromCon
         pwChars[i] = pw[i];
     }
 
-    (*env)->ReleaseCharArrayElements(env, pwCharArray, pwChars, 0);
+    if( pwIsCopy ) {
+        /* copy back the changes */
+        (*env)->ReleaseCharArrayElements(env, pwCharArray, pwChars, JNI_COMMIT);
+        /* clear the copy */
+        memset(pwChars, 0, pwlen);
+        /* release the copy */
+        (*env)->ReleaseCharArrayElements(env, pwCharArray, pwChars, JNI_ABORT);
+    } else {
+        /* pwChars is not a copy, so this should be a no-op, but we include
+         * it anyway */
+        (*env)->ReleaseCharArrayElements(env, pwCharArray, pwChars, 0);
+    }
+    pwChars = NULL;
 
     /***************************************************
      * Construct a new Password from the char array
@@ -682,13 +698,6 @@ finish:
         memset(pw, 0, strlen(pw));
         PR_Free(pw);
     }
-    if(pwChars != NULL) {
-        PR_ASSERT(pwCharArray != NULL);
-        if(pwIsCopy || password==NULL) {
-            /* clear if it's a copy, or if we are going to clear the array */
-            memset(pwChars, 0, pwlen);
-        }
-    }
     return password;
 }
 
@@ -781,7 +790,6 @@ jbyteArray
 JSS_SECItemToByteArray(JNIEnv *env, SECItem *item)
 {
     jbyteArray array=NULL;
-    jbyte* bytes=NULL;
 
     PR_ASSERT(env!=NULL && item!=NULL);
     PR_ASSERT(item->len >= 0);
@@ -793,19 +801,9 @@ JSS_SECItemToByteArray(JNIEnv *env, SECItem *item)
         goto finish;
     }
 
-    bytes = (*env)->GetByteArrayElements(env, array, NULL);
-    if(bytes == NULL) {
-        ASSERT_OUTOFMEM(env);
-        array = NULL; /* so the caller knows there was an error */
-        goto finish;
-    }
-
-    memcpy(bytes, item->data, item->len);
+    (*env)->SetByteArrayRegion(env, array, 0, item->len, item->data);
 
 finish:
-    if(bytes!=NULL) {
-        (*env)->ReleaseByteArrayElements(env, array, bytes, 0);
-    }
     return array;
 }
 /***********************************************************************
